#ifndef SURFACE_H_
#define SURFACE_H_

#include "vertex.h"
#include "material.h"
#include "triangle.h"

/*! \class Surface
\brief A class representing a triangular mesh.

\author Tom Fabin
\version 0.9
\date 2012-2018
*/
class Surface
{
public:
	//! Vchoz konstruktor.
	/*!
	Inicializuje vechny sloky st na hodnotu nula.
	*/
	Surface();

	//! Obecn konstruktor.
	/*!
	Inicializuje vertex podle zadanch hodnot parametr.

	\param name nzev plochy.
	\param n poet trojhelnk tvocch s.
	*/
	Surface( const std::string & name, const size_t n );

	//! Destruktor.
	/*!
	Uvoln vechny alokovan zdroje.
	*/
	~Surface();	

	//! Vrt poadovan trojhelnk.
	/*!
	\param i index trojhelnka.
	\return Trojhelnk.
	*/
	Triangle & get_triangle( const size_t i );

	//! Vrt pole vech trojhelnk.
	/*!	
	\return Pole vech trojhelnk.
	*/
	Triangle * get_triangles();

	//! Vrt nzev plochy.
	/*!	
	\return Nzev plochy.
	*/
	std::string get_name();

	//! Vrt poet vech trojhelnk v sti.
	/*!	
	\return Poet vech trojhelnk v sti.
	*/
	size_t no_triangles();

	//! Vrt poet vech vrchol v sti.
	/*!	
	\return Poet vech vrchol v sti.
	*/
	size_t no_vertices();
	
	//! Nastav materil plochy.
	/*!	
	\param material ukazatel na materil.
	*/
	void set_material( Material * material );

	//! Vrt ukazatel na materil plochy.
	/*!	
	\return Ukazatel na materil plochy.
	*/
	Material * get_material() const;

protected:

private:
	size_t n_{ 0 }; /*!< Poet trojhelnk v sti. */
	Triangle * triangles_{ nullptr }; /*!< Trojhelnkov s. */

	std::string name_{ "unknown" }; /*!< Nzev plochy. */

	//Matrix4x4 transformation_; /*!< Transforman matice pro pechod z modelovho do svtovho souadnho systmu. */
	Material * material_{ nullptr }; /*!< Materil plochy. */
};

/*! \fn Surface * BuildSurface( const std::string & name, std::vector<Vertex> & face_vertices )
\brief Sestaven plochy z pole trojic vrchol.
\param name nzev plochy.
\param face_vertices pole trojic vrchol.
*/
Surface * BuildSurface( const std::string & name, std::vector<Vertex> & face_vertices );

#endif
