#ifndef UTILS_H_
#define UTILS_H_

#define MAT_ELEM( mat, type, x, y ) reinterpret_cast<type *>( ( mat ).data + \
	( mat ).step * ( y ) + ( mat ).elemSize() * ( x ) )

// A macro to disallow the copy constructor and operator= functions
// This should be used in the private: declarations for a class
#define DISALLOW_COPY_AND_ASSIGN( type_name ) \
  type_name( const type_name & ); \
  void operator=( const type_name & )

#define SAFE_DELETE( p ) { \
	if ( p != NULL ) \
	{ \
		delete p; \
		p = NULL; \
	} \
}

#define SAFE_DELETE_ARRAY( p ) { \
	if ( p != NULL ) \
	{ \
		delete [] p; \
		p = NULL; \
	} \
}

/*! \fn float template<typename T> void SafeDeleteVectorItems( std::vector<T> v )
\brief Dealokuje vechny prvky typu T vektoru v.
\param v Standardn vektor.
*/
template<typename T> void SafeDeleteVectorItems( std::vector<T> v )
{
	while ( v.size() > 0 )
	{
		T item = v.back();
		v.pop_back();
		SAFE_DELETE( item );
	}
}

namespace utils
{
	/*! \fn void swap( T & a, T & b )
	\brief Prohod hodnoty a, b.
	\param a Prvni T hodnota.
	\param b Druha T hodnota.
	*/
	template<typename T> void swap( T & a, T & b )
	{
		const T tmp = a;
		a = b;
		b = tmp;
	}
}

/*! \fn float Random( const float range_min, const float range_max )
\brief Vrt pseudonhodn slo s normlnm rozdlenm v intervalu <range_min, range_max).
\param range_min Doln mez intervalu.
\param range_max Horn mez intervalu.
\return Pseudonhodn slo.
*/
float Random( const float range_min = 0.0f, const float range_max = 1.0f );

/*! \fn long long GetFileSize64( const char * file_name )
\brief Vrt velikost souboru v bytech.
\param file_name pln cesta k souboru
*/
long long GetFileSize64( const char * file_name );

/*! \fn void PrintTime( double t )
\brief Vytiskne na stdout as ve formtu Dd:Mm:Ss.
\param t as v sekundch.
*/
void PrintTime( double t, char * buffer );

/*! \fn std::string TimeToString( const double t )
\brief Pevede as v sekundch do etzce ve formtu Dd:Mm:Ss nebo Ss.m nebo m.u
\param t as v sekundch.
*/
std::string TimeToString( const double t );

/*! \fn char * LTrim( char * s )
\brief Oee etzec o bl znaky zleva.
\param s ukazatel na etzec
\return Ukazatel na novou pozici v tomt etzci
*/
char * LTrim( char * s );

/*! \fn char * RTrim( char * s )
\brief Oee etzec o bl znaky zprava.
\param s ukazatel na etzec
\return Ukazatel na novou pozici v tomt etzci
*/
char * RTrim( char * s );

/*! \fn char * Trim( char * s )
\brief Oee etzec o bl znaky z obou stran.
\param s ukazatel na etzec
\return Ukazatel na novou pozici v tomt etzci
*/
char * Trim( char *s );

#endif
